// RS232.cpp: implementation of the CRS232 class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RS232.h"
#include "slip.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRS232::CRS232()
{
	m_hWnd = NULL;

	osRead.hEvent = NULL;
	osWrite.hEvent = NULL;

	idComDev = NULL;
	fConnected = FALSE;
}

CRS232::~CRS232()
{
	CloseConnection();
	CloseComPort();
/*
	if(fConnected == TRUE) {
		CloseHandle( osWrite.hEvent ); 
		CloseHandle( osRead.hEvent ); 

		CloseHandle( idComDev) ; 
		fConnected = FALSE ; 
	}
*/
}

void CRS232::SetCommPort(HWND hWnd, int port, DWORD baudrate, int databits, int parity, int stopbits)
{
	m_hWnd = hWnd;

	m_port = port;
	m_baudrate = baudrate;
	m_databits = databits;
	m_parity = parity;
	m_stopbits = stopbits;
}


// CComm message handlers 
//CommWatchProc() 
// ϴ ν,  Ͱ   ϴ 
//ƾ.  ƾ OpenComPort Լ  ν . 
//OpenComPort Լ  

#define MAXBLOCK  100
DWORD CommWatchProc(LPVOID lpData) 
{ 
	DWORD dwEvtMask ; 
	OVERLAPPED os ; 
	CRS232* npComm = (CRS232*) lpData ; 
	char InData[MAXBLOCK + 1]; 
	int nLength ; 

	//idCommDev ڵ鿡 ƹ Ʈ  پ    
	if ( npComm == NULL)
		return (DWORD)(-1); 

	memset( &os, 0, sizeof( OVERLAPPED ) ) ; 
	os.hEvent = CreateEvent( NULL, // no security 
		TRUE, // explicit reset req 
		FALSE, // initial event reset 
		NULL ) ; // no name 

	if ( os.hEvent == NULL ) { 
		MessageBox( NULL, "Failed to create event for thread!", "comm Error!", 
		MB_ICONEXCLAMATION | MB_OK ) ; 
		return ( FALSE ) ; 
	} 

	if (!SetCommMask(npComm->idComDev, EV_RXCHAR )) 
		return ( FALSE ) ; 

	while (npComm->fConnected ) { 
		dwEvtMask = 0 ; 

		WaitCommEvent(npComm->idComDev, &dwEvtMask, NULL ); 
		if ((dwEvtMask & EV_RXCHAR) == EV_RXCHAR) { 
			do { 
				if (nLength = npComm->ReadCommBlock((LPSTR) InData, MAXBLOCK )) { 
					SendMessage(npComm->m_hWnd,WM_RECEIVEDATA ,(WPARAM)nLength,(LPARAM)&InData); 
				} 
			}while ( nLength > 0 ) ; 
		} 
	} 
	CloseHandle( os.hEvent ) ; 
	return( TRUE ) ; 
} 

int CRS232::OpenComPort()
{
	char szPort[ 15 ] ; 
	COMMTIMEOUTS CommTimeOuts ; 

	if(m_hWnd == NULL)
		return(FALSE);

    if(!(COM1 <= m_port && m_port <= COM6))
		return(FALSE);
	 
	osWrite.Offset = 0 ; 
	osWrite.OffsetHigh = 0 ; 
	osRead.Offset = 0 ; 
	osRead.OffsetHigh = 0 ; 
	 
	osRead.hEvent = CreateEvent( NULL, TRUE, FALSE, NULL ); 
	if (osRead.hEvent == NULL) 
		return FALSE ; 
	 
	osWrite.hEvent = CreateEvent( NULL, TRUE, FALSE, NULL ); 
	if (NULL == osWrite.hEvent) {  
		CloseComPort();
		return FALSE; 
	} 
	 
	wsprintf( szPort, "COM%d", m_port ) ; 
	// COMM device   Ѵ. 
	if ((idComDev = CreateFile( szPort, GENERIC_READ | GENERIC_WRITE, 
			0, // exclusive access 
			NULL, // no security attrs 
			OPEN_EXISTING, 
			FILE_FLAG_OVERLAPPED, // overlapped I/O 
			NULL )) == INVALID_HANDLE_VALUE) {
		
		idComDev = NULL;
		CloseComPort();
		return ( FALSE ) ; 
	}

	m_PrevDCB.DCBlength = sizeof( DCB ) ; 
	GetCommState( idComDev, &m_PrevDCB ) ;//dcb ⺻ ޴´. 

	 
	//Ʈ ͸ ȯϴ  char  ⺻ . 
	SetCommMask( idComDev, EV_RXCHAR ) ; 
	SetupComm( idComDev, 4096, 4096 ) ; 
	//̽ Ⱑ  𸣴ϱ  ûҸ . 
	PurgeComm( idComDev, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR ) ; 

	CommTimeOuts.ReadIntervalTimeout = 0xFFFFFFFF ; 
	CommTimeOuts.ReadTotalTimeoutMultiplier = 0 ; 
	CommTimeOuts.ReadTotalTimeoutConstant  = (256/120 + 1) * 1000 ;   // 1000
	CommTimeOuts.WriteTotalTimeoutMultiplier = 0 ;  
	CommTimeOuts.WriteTotalTimeoutConstant = (256/120 +1) * 1000 ;  // 1000 
	SetCommTimeouts( idComDev, &CommTimeOuts ) ; 
	 
	if (SetupConnection())  { 
		//ν CommWatchProc ϴϱ ߿ Ͱ Դٰ 
		//ϸ   CommWatchProc Ѵ. 
		AfxBeginThread((AFX_THREADPROC)CommWatchProc,(LPVOID)this); 
	    return TRUE ; 
	} 
	else { 
		CloseComPort();

		return FALSE;
	} 
} 

int CRS232::CloseComPort()
{
	if(fConnected) 
		CloseConnection();
	
	if(osWrite.hEvent != NULL) {
		CloseHandle( osWrite.hEvent ); 
		osWrite.hEvent = NULL;
	}

	if(osRead.hEvent != NULL) {
		CloseHandle( osRead.hEvent ); 
		osRead.hEvent = NULL;
	}
 
	if(idComDev != NULL && idComDev != INVALID_HANDLE_VALUE) {
		CloseHandle(idComDev) ; 
		idComDev = NULL;
	}

	return TRUE;
}

BOOL CRS232::SetupConnection()
{
	DCB  dcb; 
     
	FillMemory(&dcb,sizeof(DCB),0);

	if(GetCommState( idComDev, &dcb )) //dcb ⺻ ޴´. 
		m_PrevDCB = dcb;
	else 
		return FALSE;

	dcb.DCBlength = sizeof(DCB) ; 
	
	dcb.BaudRate = m_baudrate;	// ӵ 
	dcb.ByteSize = m_databits;	// Ʈ 
	dcb.Parity = m_parity;		//иƼ üũ 
	dcb.StopBits = m_stopbits;	// Ʈ 

	dcb.fOutxDsrFlow =0 ;//Dsr Flow 
	dcb.fDtrControl = DTR_CONTROL_DISABLE ;//Dtr Control 
	dcb.fOutxCtsFlow = 0 ;//Cts Flow 
	dcb.fRtsControl = RTS_CONTROL_DISABLE ; //Ctr Control 
//	dcb.fInX = dcb.fOutX = 1 ; //XON/XOFF   
 	dcb.XonChar = ASCII_XON ; 
 	dcb.XoffChar = ASCII_XOFF ; 
//	dcb.XonLim = 100 ; 
//	dcb.XoffLim = 100 ; 
	dcb.fBinary = TRUE ; 
	dcb.fParity = TRUE ; 

	if(SetCommState( idComDev, &dcb )) {  // Dcb  
		fConnected = TRUE ;               //Ǿٰ  . 
		return(TRUE);
	}
	else {
		SetCommState(idComDev,&m_PrevDCB);
		fConnected = FALSE ; 
		return(FALSE);
	}
}

// ݴ´. 

BOOL CRS232::CloseConnection()
{
	// set connected flag to FALSE 
	fConnected = FALSE; 
	// disable event notification and wait for thread 
	// to halt 
	SetCommMask( idComDev, 0 ); 

	EscapeCommFunction( idComDev, CLRDTR ); 
	PurgeComm( idComDev, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR ) ; 

  	SetCommState(idComDev,&m_PrevDCB);

	return ( TRUE ) ; 
} 

//Ʈκ ͸ д´. 
int CRS232::ReadCommBlock(LPSTR lpszBlock, int nMaxLength)
{
	BOOL fReadStat ; 
	COMSTAT ComStat ; 
	DWORD dwErrorFlags; 
	DWORD dwLength; 

	if(idComDev == NULL) 
		return(FALSE);

	// only try to read number of bytes in queue 
	ClearCommError( idComDev, &dwErrorFlags, &ComStat ) ; 
	dwLength = min( (DWORD) nMaxLength, ComStat.cbInQue ) ; 
	if (dwLength > 0) { 
		fReadStat = ReadFile( idComDev, lpszBlock,dwLength, &dwLength, &osRead); 

		if (!fReadStat) { 
			//̰  ִ. 
			// ReadFile  Ͱ    fReadState  
			// ڵ带 Ѵ.        
			// Ұϴ. ,  ش޶ ޽ ִ  . 
			;
		} 
	} 
	return ( dwLength ) ; 
}

//DEL int CRS232::GetChar()
//DEL {
//DEL 	static int flag = 0;
//DEL 	static int len,escflag;
//DEL 	BYTE data;
//DEL 
//DEL 	if(flag == 0) {
//DEL 		escflag = 0;
//DEL 
//DEL 		data = FEND;
//DEL 		len = rand() % 10 + 1;
//DEL 		flag = 1;
//DEL 	}
//DEL 	else if(flag == 1) {
//DEL 		if(escflag != 0) {
//DEL 			data = (BYTE)escflag;
//DEL 			escflag = 0;
//DEL 
//DEL 			len--;
//DEL 		}
//DEL 		else {
//DEL 		    data = (BYTE)rand(); 
//DEL 
//DEL 		    if(data == FEND) {
//DEL 				data = FESC;
//DEL 				escflag = TFEND;
//DEL 			}
//DEL 			else if(data == FESC) {
//DEL 				data = FESC;
//DEL 				escflag = TFESC;
//DEL 			}
//DEL 			else {
//DEL 				len--;
//DEL 			}
//DEL 
//DEL 		    if(len == 0)  flag = 2;
//DEL 		}
//DEL 	}
//DEL 	else if(flag == 2) {
//DEL 		data = FEND;
//DEL 		flag = 0;
//DEL 	}
//DEL 	else {
//DEL         flag = 0;
//DEL 	}
//DEL 
//DEL 
//DEL // int CRS232::ReadCommBlock(LPSTR lpszBlock, int nMaxLength)
//DEL 
//DEL 	SendMessage(m_hWnd,WM_RECEIVEDATA,(WPARAM)1,(LPARAM)&data);
//DEL 	return(flag);
//DEL }

BOOL CRS232::WriteCommBlock(LPSTR lpByte, DWORD dwBytesToWrite)
{
	BOOL fWriteStat ; 
	DWORD dwBytesWritten ; 

	if(idComDev == NULL) 
		return(FALSE);

	fWriteStat = WriteFile( idComDev, lpByte, dwBytesToWrite,&dwBytesWritten, &osWrite); 
	if (!fWriteStat) { 
		//Ʈ ͸    ̴. 
		//   ұ. װ  ̴. 
		//ٽ   ͼ ϸ ȴ. 
		//׷     ִٴ  . 
		int errcode;

		if(GetLastError() != ERROR_IO_PENDING) {
			return(FALSE);
		}
		else {
			if(WaitForSingleObject(osWrite.hEvent,INFINITE) == WAIT_OBJECT_0) {
				if(!(GetOverlappedResult(idComDev,&osWrite,&dwBytesWritten,FALSE))) {
			        errcode = TransmitCommChar(idComDev,*lpByte);
					return(FALSE);
				}
				else
					return(TRUE);
			}
			else
				return(FALSE);

		}
	} 
	return ( TRUE ) ; 
}

static int  s_TxSize = 0;
static char s_TxBuff[(HEADER_SIZE + INFO_BUFFERSIZE + CRC_SIZE) * 2];

void CRS232::PutStart()
{
	s_TxSize = 0;
}

BOOL CRS232::PutChar(BYTE data)
{
	s_TxBuff[s_TxSize++] = data;
	return(TRUE);
}

void CRS232::PutEnd()
{
	if(WriteCommBlock((LPSTR)&s_TxBuff,( DWORD )s_TxSize)) { 
		SendMessage(m_hWnd,WM_TRANSMITTDATA ,(WPARAM)s_TxSize,(LPARAM)&s_TxBuff);

		s_TxSize = 0;
	}
	else {
		s_TxSize = 0;
	}
}

BOOL CRS232::EnableRxEvent()
{
	DWORD EvtMask;

	if(GetCommMask(idComDev,&EvtMask)) {
        EvtMask = EvtMask | EV_RXCHAR;

		SetCommMask(idComDev,EvtMask);
		return(TRUE);
	}
	else
		return(FALSE);
}

BOOL CRS232::DisableRxEvent()
{
	DWORD EvtMask;

	if(GetCommMask(idComDev,&EvtMask)) {
        EvtMask = EvtMask | ~(EV_RXCHAR);

		SetCommMask(idComDev,EvtMask);
		return(TRUE);
	}
	else
		return(FALSE);

}

BOOL CRS232::EnableTxEvent()
{
	DWORD EvtMask;

	if(GetCommMask(idComDev,&EvtMask)) {
        EvtMask = EvtMask | EV_TXEMPTY;

		SetCommMask(idComDev,EvtMask);
		return(TRUE);
	}
	else
		return(FALSE);
}

BOOL CRS232::DisableTxEvent()
{
	DWORD EvtMask;

	if(GetCommMask(idComDev,&EvtMask)) {
        EvtMask = EvtMask & ~(EV_TXEMPTY);

		SetCommMask(idComDev,EvtMask);
		return(TRUE);
	}
	else
		return(FALSE);
}

