// SLIP.h: interface for the CSLIP class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SLIP_H__0C0939A2_43C0_11D4_84D9_0050DA8E82D6__INCLUDED_)
#define AFX_SLIP_H__0C0939A2_43C0_11D4_84D9_0050DA8E82D6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "RS232.h"

typedef  unsigned char    uchar;
typedef  unsigned int     uint;
typedef  unsigned short   uint16;
typedef  unsigned long    uint32;

#define  FEND     0xC0
#define  FESC     0xDB
#define  TFEND    0xDC
#define  TFESC    0xDD

/* frame type define */
#define RNR_FRAME                         0x00
#define RR_FRAME                          0x01
#define INVALID_ADDRESS_FRAME             0x02
#define INVALID_FRAME_TYPE                0x03
#define INVALID_FRAME_LENGTH              0x04
#define BAD_CCITT16                       0x05

#define ACK_FRAME                         0x10
#define TIME_SET                          0x11
#define UART_RESET                        0x12
#define UART_CONTROL                      0x13
#define QUEUE_INITIAL                     0x14
#define LED_ONOFF                         0x15
#define TEXT_MSG                          0x16
#define NAK_FRAME                         0x1F

#define ECHO_REQUEST                      0x20
#define ECHO_REPLY                        0x30

/* frame construction */

#define INFO_MAXSIZE                      256
#define HEADER_SIZE                       5    /* DA:2 +SA:2 + FT:1 */
#define INFO_BUFFERSIZE                   INFO_MAXSIZE
#define CRC_SIZE                          2
#define FRAME_BUFFSIZE                    (HEADER_SIZE + INFO_BUFFERSIZE + CRC_SIZE + 2)

#define SA_ADDRESS   0x0182
#define DA_ADDRESS   0x0042

#define SETHEADER(frame,da,sa,ft) frame.InfoFrame.DA = da, frame.InfoFrame.SA = sa, frame.InfoFrame.FT = ft

typedef struct {
    uint16 low;
	uint16 high;
} tagint32;

#define SetInt32(tag,value)   tag.low = (uint16)(value), tag.high = (value) / 0x10000L
#define GetInt32(tag)         (tag).high * 0x10000L + (tag).low

#define DefHeader()    \
       uint16 DA;      \
	   uint16 SA;      \
	   uchar  FT;

   struct TagInfoFrame {
       DefHeader()

       uchar  info_ccitt[INFO_BUFFERSIZE + CRC_SIZE];
	} ;

   struct TagNakFrame {
       DefHeader()

       uchar  RxFT;
       uint16 ErrCode;
   };

   struct TagLedOnOffFrame {
	   DefHeader()

	   uchar  LedNo;
	   uchar  LedOnOff;
   };

   struct TagTextFrame {
	   DefHeader()

       uchar  TextMsg[INFO_BUFFERSIZE + CRC_SIZE];      
   };

typedef union tagFrame {   
   struct TagInfoFrame      InfoFrame;
   struct TagNakFrame       NakFrame;
   struct TagLedOnOffFrame  LedOnOffFrame;
   struct TagTextFrame      TextFrame;

   uchar  FrameStr[HEADER_SIZE + INFO_BUFFERSIZE + CRC_SIZE];
} FRAME;

#define InfoSize(x)      (sizeof(struct x) - HEADER_SIZE)

class CSLIP : public CRS232  
{
public:
	int SendChar(BYTE data);
	int ReceiveFrame(BYTE rxdata);
	int SendFrame(int infolen,FRAME *frame = (FRAME *)0);
	CSLIP();
	virtual ~CSLIP();

private:
	WORD m_TxCRC;
	WORD m_RxCRC;
};

#endif // !defined(AFX_SLIP_H__0C0939A2_43C0_11D4_84D9_0050DA8E82D6__INCLUDED_)
